/*
    This is the source file for a very simple "library" that exports
    three functions.

    After this file has been compiled, its obj file can be linked in
    two different ways.

    First, this file's obj file can be statically linked to any obj
    file that imports the functions that this file exports. For
    example, to statically link these functions to the obj file
    my-library-test.obj, use the following command.

      lcclnk my-library-test.obj my-library.obj

    Note that when these functions are statically linked to an executable
    exe file, these functions become part of the executable.


    On the other hand, the linker can use this file's obj file to create
    a DLL file. Then the DLL file can be dynamically loaded by any exe file
    that wants to use the functions that this file exports. To create a DLL
    file from this file's obj file, use the following command.

      lcclnk -DLL -nounderscores my-library.obj my-library.def

    The default output file will be called my-library.dll. In addition,
    the linker will also create a lib file called my-library.lib.

    After the linker has created the DLL file, there are two ways that an exe
    file can dynamically load and make use of the DLL. A program can either
    "implicitly" load the DLL at the program's loadtime or a program can
    "explicitly" load the DLL later during the program's runtime.

    A program implicitly loads a DLL if the program has been linked to the DLL's
    lib file. For example, here is how to link the program my-library-test to the
    DLL version of this "library".

      lcclnk my-library-test.obj my-library.lib

    Note that when the DLL version of this "library" is linked to an executable
    exe file, the library functions are not part of the executable. Only a very
    small stub function is placed in the executable for each library function.

    Explicitly loading a DLL during runtime will be demonstrated in a separate file.

    NOTE: See the end of this file for commands to use gcc or Microsoft's linker
          to create and use a dll.
*/
#include <stdio.h>
#include "my-library.h"  // not required but it's a good thing to include this
                         // header to make sure the declarations in the header
                         // match up with the definitions in this file
int f(int x)
{
   printf("  f called with parameter x = %d\n", x);
   printf("  function f is loaded at address %#.8x\n", (int)&f);

   return x+1;
}

int g(int x, int y)
{
   printf("  g called with parameters x, y = %d, %d\n", x, y);
   printf("  the address of function g is %#.8x\n", (int)&g);

   return x+2;
}

int h(int x, int y, int z)
{
   printf("  g called with parameters x, y, z = %d, %d, %d\n", x, y, z);
   printf("  the address of function h is %#.8x\n", (int)&h);

   return x+3;
}

/*
   Here are the command lines to use Microsoft's Visual Studio 6
   linker (link.exe) to create a dll from this program's obj file
   and then link an obj file with the dll.

   link /DLL /DEF:my-library.def my-library.obj
   link my-library-test.obj my-library.lib

   Note 1: The above two commands do not put debugging information into
           the dll. The following commands should provide all the needed
           debugging information.

   link /DEBUG /PDB:NONE /DLL /DEF:my-library.def my-library.obj
   link /DEBUG /PDB:NONE my-library-test.obj my-library.lib

   Note 2: These command lines assume that you have executed the batch file
           VCVARS32.BAT.
*/

/*
  After compiling my-library.c and my-library-test.c with gcc, here are the gcc
  commands for creating my-library.dll and linking an obj file with the dll.

  gcc -shared -Wl,--out-implib,my-library.lib -o my-library.dll my-library.obj my-library.def
  gcc -o my-library-test.exe my-library-test.obj my-library.lib

*/
